'use strict';

const sequence = require('../sequence');
const parallel = require('../parallel');
const co = require('co');
const _ = require('lodash');

const wrapForTasks = (runner) => {
  return (tasks) => {
    if (!tasks) return [];
    tasks = Array.isArray(tasks) ? tasks : [tasks];

    return co(function* () {
      const commands = _.map(tasks, 'command');
      const results = yield runner(commands);
      const completedTasks = results.length;
      const createState = (index) => {
        return index < completedTasks ? !(results[index] instanceof Error) ? 'resolved' : 'rejected' : 'skipped';
      };

      return tasks.map((task, index) => ({
        id: task.id,
        result: results[index],
        state: createState(index)
      }));
    });
  };
};

module.exports = {
  sequence: wrapForTasks(sequence),
  parallel: wrapForTasks(parallel)
};

